function r = getTurtle()
% getTurtle: Sets up basic L-System letters containing turtle commands.
%
% The following letters are interpreted by a 2D and 3D turtle:
% field    : name  :  description
% --------------------------------------
% F        :  'F' : Forward (length)
% Color    :  'C' : Color (color) 
% LW       :  '#' : Line width (width) 
% Push     :  '[' : Push turtle state
% Pop      :  ']' : Pop turtle state
% 2D turtle
% Plus     :  '+' : Rotate ccw (delta)
% Minus    :  '-' : Rotate cw (delta)
% PlusMinus:  '1' : Randomly left or right rotation (*)
% Bend2D   :  'b' : rule for root tip deflection (*)
% 3D turtle
% TurnL    :  '+' : Turn left by angle delta
% TurnR    :  '-' : Turn right by angle delta
% PitchD   :  '&' : Pitch down by angle delta
% PitchU   :  '^' : Pitch up by angle delta
% RollL    :  '\' : Roll left by angle delta
% RollR    :  '/' : Roll right by angle delta.
% TurnAround: '|' : Turn around
% BranchRotate:'3': Randomly roll and turn left by angle delta (*)
% BranchRotate2D:'4': Randomly roll and randomly turn left by angle
%       rand*delta (*)
% Bend3D   :  'b' : rule for root tip deflection (*) 
%
% (*) this commands are not interpreted by the turtle but replaced by more
%       basic turtle commands.
%
% Parameters:
% r        :A structure containing the basic L-System letters.
%
% Copyright 2008-2009 Daniel Leitner. See license.txt for details.
%

global rules;

id = @(x,dt) {x};

rules.F.name = 'F';
rules.F.length = 1;
rules.F.apply = id;
rules.F.time = 0;

rules.Color.name = 'C';
rules.Color.color = [.5 .4 0];
rules.Color.apply = id;

rules.LW.name = '#';
rules.LW.width = 1;
rules.LW.apply = id;

rules.Push.name = '[';
rules.Push.apply = id;

rules.Pop.name = ']';
rules.Pop.apply = id;

% 2D

rules.Plus.name = '+';
rules.Plus.delta = pi/2;
rules.Plus.apply = id;

rules.Minus.name = '-';
rules.Minus.delta = pi/2;
rules.Minus.apply = id;

rules.PlusMinus.name = '1';
rules.PlusMinus.delta = pi/2;
rules.PlusMinus.apply = @(x,dt) rulePlusMinus(x);

rules.Bend.name = 'b';
rules.Bend.sigma = pi/30;
rules.Bend.apply = @(x,y,dx) ruleBend2D(x,y);

% 3D

rules.TurnL.name='+';
rules.TurnL.delta = pi/3;
rules.TurnL.apply = id;

rules.TurnR.name='-';
rules.TurnR.delta = pi/3;
rules.TurnR.apply = id;

rules.PitchD.name='&';
rules.PitchD.delta = pi/3;
rules.PitchD.apply = id;

rules.PitchU.name='^';
rules.PitchU.delta = pi/3;
rules.PitchU.apply = id;

rules.RollL.name='\';
rules.RollL.delta = pi/3;
rules.RollL.apply = id;

rules.RollR.name='/';
rules.RollR.delta = pi/3;
rules.RollR.apply = id;

rules.TurnAround.name='|';
rules.TurnAround.apply = id; 

rules.BranchRotate.name = '3';
rules.BranchRotate.delta = pi/3;
rules.BranchRotate.sigma = 0;
rules.BranchRotate.apply = @(x,dt) ruleBranchRotate(x);

rules.BranchRotate2D.name = '4';
rules.BranchRotate2D.delta = pi/3;
rules.BranchRotate2D.sigma = 0;
rules.BranchRotate2D.apply = @(x,dt) ruleBranchRotate2D(x);

rules.Bend3D.name = 'b';
rules.Bend3D.sigma = pi/30;
rules.Bend3D.apply = @(x,y,dx) ruleBend3D(x,y);

r=rules;



function new = rulePlusMinus(x)
global rules;
if (rand>0.5) 
    t = rules.Plus;
else
    t = rules.Minus;
end
t.delta = x.delta;
new={t};



function new = ruleBend2D(x,tip)
global rules;
delta = rand*x.sigma;
if (rand>0.5) 
    t = rules.Plus;
else
    t = rules.Minus;
end
t.delta = delta;
new ={t};



function new = ruleBranchRotate(x)
global rules;
r = rules.RollL;
r.delta = 2*pi*rand;
t = rules.TurnL;
t.delta = x.delta; 
new = {r,t};



function new = ruleBranchRotate2D(x)
global rules;
r = rules.RollL;
if rand>0.5 
    r.delta = pi/2;
else
    r.delta = 3*pi/2;
end
t = rules.TurnL;
t.delta = x.delta; 
new = {r,t};



function new = ruleBend3D(x,tip)
global rules;
r = rules.RollL;
    if (x.sigma==0) 
        r.delta =0 ;
    else
        r.delta= rand*2*pi;
    end
t = rules.TurnL;
t.delta = rand*x.sigma;
new = {r,t};




