function new = ruleCreateHighestOrderRoot(x, r, rs, k, ks, theta, thetas, ...
    a, as, color, deflect, dx, dt,lifetime,userlambda)
% ruleCreateHighestOrderRoot: Creates a root with no succesive orders.
%
% Parameters:
% x         :letter that is replaced by the production rule
% r         :initial growth speed (cm/day)
% rs        :standard deviation of initial growth speed
% k         :maximal length (cm)
% ks        :standard deviation of maximal length
% theta     :angle betwenn branch and preceeding branch
% thetas    :standard deviation of angle
% a         :radius (cm)
% as        :standard deviation of radius
% color     :color ([r g b])
% deflect   :production rule for root tip deflection
% dx        :spatial discretization along the root (cm)
% dt        :time step
% (lifetime):time the root lives. Default lifetime=inf
% (userlambda): user defined growth fucntion (default=negative exponential)
%
% new       :resulting L-system letter
%
% See also: ruleDefinition, ruleCreateRoot, ruleSectionGrowth
%
% Copyright 2008-2009 Daniel Leitner. See license.txt for details.
%

r = r + randn*rs; % apply standard deviation s
k = k + randn*ks;
theta = theta + randn*thetas;
a = a + randn*as;
theta = max(theta,0); % check if numbers are non-negative
a = max(a, 0.0001);
k = max(k,0);

if (nargin<14)
    lifetime=inf;        
end

lambda0 = @(t,y) k*(1-exp(-(r/k)*t));
if (nargin<15)
    lambda = lambda0;
else
    lambda = userlambda;
end

if r>0 && k>(dx/10)

    rules = getTurtle;
    
    t0 = 0;
    tend = -k/r*log(dx/10/k);
    branch.name = 'G';
    branch.t = 0;
    branch.l = 0;
    if isfield(x,'xT')
        branch.xT = x.xT;
        branch.yT = x.yT;
        branch.zT = x.zT;
        branch.wT = x.wT;
        branch.cT = x.cT;
        branch.R = x.R;
    else
        disp('ruleCreateHighestOrderRoot: no label');
    end
    branch.apply = @(x,dt) ruleSectionGrowth(x,lambda,lambda0(t0,branch),lambda0(tend,branch), ...
        t0, tend, rules.TurnAround,deflect,dx,dt);
    
    die = {};
    if (lifetime~=inf)
        cdie = rules.Color;
        cdie.color = [1,1,1];
        die.name='d';
        die.apply = @(x,dt) ruleDelay(x,lifetime,cdie,dt);
    end
    rules.LW.width=2*a;
    rules.BranchRotate.delta = theta;
    rules.Color.color=color;
    br = rules.BranchRotate.apply(rules.BranchRotate);
    new = [{rules.Push}, {rules.LW}, {rules.Color}, die, br];
    branch = labelTip(branch, new);

    branch = branch.apply(branch,dt);

    new = [new, branch, {rules.Pop}];

else

    new = {};

end
